/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef COLOR_HPP
#define COLOR_HPP

#include <iostream>
#include <exception_cpe.hpp>


namespace cpe
{
class vec3;
typedef float color_component;

    /** \brief Container class for (r,g,b) color. Each component is between [0,1] */
    class color
    {
    public:


        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTORS
        // ********************************************* //
        // ********************************************* //

        /** \brief empty constructor (0,0,0) */
        color();
        /** \brief direct constructor with gray value (r,b,g)=(gray,gray,gray) */
        color(color_component gray);
        /** \brief direct constructor with (r,b,g) value */
        color(color_component r,color_component g,color_component b);
        color(const vec3& c);

        // ********************************************* //
        // ********************************************* //
        //  ACCESS COLOR VALUE
        // ********************************************* //
        // ********************************************* //

        /** \brief get red value */
        color_component r() const;
        /** \brief get red value */
        color_component& r();
        /** \brief get green value */
        color_component g() const;
        /** \brief get green value */
        color_component& g();
        /** \brief get blue value */
        color_component b() const;
        /** \brief get blue value */
        color_component& b();

        // ********************************************* //
        // ********************************************* //
        //  COLOR MANIPULATION
        // ********************************************* //
        // ********************************************* //


        /** \brief return clamp the color (between 0 and 1) */
        void clamp();



    private:

        // ********************************************* //
        // ********************************************* //
        //  INTERNAL STORAGE
        // ********************************************* //
        // ********************************************* //

        /** \brief internal red component */
        color_component internal_r;
        /** \brief internal green component */
        color_component internal_g;
        /** \brief internal blue component */
        color_component internal_b;

    };

    color& operator+=(color& c0,const color& c1);
    color& operator-=(color& c0,const color& c1);
    color operator+(const color& c0,const color& c1);
    color operator-(const color& c0,const color& c1);
    color& operator*=(color& c0,float alpha);
    color& operator*=(color& c0,const color& c1);
    color& operator/=(color& c0,float alpha);
    color operator*(const color& c0,float alpha);
    color operator*(const color& c0,const color& c1);
    color operator*(float alpha,const color& c0);
    color operator/(const color& c0,float alpha);


    // ********************************************* //
    // ********************************************* //
    //  I/O
    // ********************************************* //
    // ********************************************* //

    /** \brief standard output */
    std::ostream& operator<<(std::ostream& stream,const color& c);



    /** \brief Exception class for color */
    class exception_color: public exception_cpe
    {
    public:
        /** \brief empty constructor */
        exception_color():exception_cpe(){}
        /** \brief constructor with error information */
        exception_color(const std::string& msg_param,const std::string& file_param,const std::string& caller_param,const int& line_param):exception_cpe(msg_param,file_param,caller_param,line_param){}
    };
}

#endif
