/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <plane.hpp>
#include <ray.hpp>

#include <cmath>

#include <intersection.hpp>

namespace cpe
{
    plane::plane(const vec3& x0_data,const vec3& normal_data):x0(x0_data),n(normal_data){}
    plane::~plane(){}


    const vec3& plane::normal() const {return n;}
    const vec3& plane::position() const {return x0;}

    bool plane::intersect(const ray& ray_data,intersection& intersection_data) const
    {
        float epsilon=1e-6;

        const vec3& ray_x0=ray_data.x0;
        const vec3& ray_u=ray_data.u;


        // ********************************************************** //
        // ********************************************************** //
        //  TO DO:
        //    Calcul d'intersection entre un rayon et une plan
        //
        // Variales:
        //  - Position initiale du rayon: ray_x0
        //  - Vecteur directeur unitaire du rayon: ray_u
        //  - Position d'un point du plan: x0
        //  - Normale unitaire au plan: n0
        //
        // Aide de syntaxe:
        //  - Calcul du produit scalaire <ray_u,n>: ray_u.dot(n);
        //  - Mise a jour de la donnee d'intersection:
        //                intersection_data.set(x_inter,n_inter,t_inter);
        //
        // ********************************************************** //
        // ********************************************************** //


        //Le code suivant est arbitraire est doit etre modifie
        float p=ray_u.dot(vec3(2*n.y()+n.x(),n.y(),n.z()));
        float t=2*p*(x0.normalized()-n).dot(n);

        if(t>0)
            return true;
        else
            return false;
    }

}
