/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef PLANE_HPP
#define PLANE_HPP


#include <object3d.hpp>
#include <vec3.hpp>

namespace cpe
{

    /** \brief Object3d of type (infinite) Plane (position+normal). Analytically compute intersection with a ray */
    class plane : public object3d
    {
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTOR
        // ********************************************* //
        // ********************************************* //

        /** \brief direct constructor (position+normal) */
        plane(const vec3& x0=vec3(),const vec3& normal=vec3(0,0,1));

        /** \brief destructor */
        virtual ~plane();

        // ********************************************* //
        // ********************************************* //
        //  Accessor
        // ********************************************* //
        // ********************************************* //

        /** \brief get the normal of the plane */
        const vec3& normal() const;

        /** \brief get the position of the plane */
        const vec3& position() const;


        // ********************************************* //
        // ********************************************* //
        //  INTERSECTION
        // ********************************************* //
        // ********************************************* //

        /** \brief Analytic intersection between a ray and the plane */
        virtual bool intersect(const ray& ray_data,intersection& intersection_data) const;

    private:

        // ********************************************* //
        // ********************************************* //
        //  INTERNAL DATA
        // ********************************************* //
        // ********************************************* //


        /** \brief internal position of the plane (not unique for a given plane) */
        const vec3 x0;
        /** \brief internal normal of the plane */
        const vec3 n;
    };
}

#endif
