/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/




#ifndef SPHERE_HPP
#define SPHERE_HPP

#include <object3d.hpp>
#include <vec3.hpp>

namespace cpe
{

    class ray;

    /** \brief Object3d of type Sphere (center+radius). Analytically compute intersection with a ray */
    class sphere : public object3d
    {
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTOR
        // ********************************************* //
        // ********************************************* //

        /** \brief direct constructor (center+radius) */
        sphere(const vec3& center=vec3(),float radius=1.0);

        /** \brief desctructor */
        virtual ~sphere();


        // ********************************************* //
        // ********************************************* //
        //  Accessor
        // ********************************************* //
        // ********************************************* //

        /** \brief get radius */
        double radius() const;

        /** \brief get center */
        const vec3& center() const;

        // ********************************************* //
        // ********************************************* //
        //  INTERSECTION
        // ********************************************* //
        // ********************************************* //

        /** \brief Analytic intersection between a ray and the sphere */
        virtual bool intersect(const ray& ray_data,intersection& intersection_data) const;

    private:

        // ********************************************* //
        // ********************************************* //
        //  INTERNAL DATA
        // ********************************************* //
        // ********************************************* //

        /** \brief internal center of the sphere */
        vec3 x0;
        /** \brief internal radius of the sphere */
        float r;
    };


}

#endif
