/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <anti_aliasing_table.hpp>

#include <assert.h>
#include <cmath>

namespace cpe
{

anti_aliasing_table::anti_aliasing_table(int N,float L)
{
    assert(N>0);

    if(N==1)
    {
        displacement.push_back(0);
        weight.push_back(1);
        return ;
    }

    for(int k=0;k<N;++k)
    {
        float u=k/(N-1.0);
        float uc=2.0*(u-0.5);

        float dL_value=L*uc;
        displacement.push_back(dL_value);
    }

    float total_w=0.0;
    for(int kx=0;kx<N;++kx)
    {
        float dx=displacement[kx];
        for(int ky=0;ky<N;++ky)
        {
            float dy=displacement[ky];

            float u=(dx*dx+dy*dy)/(L*L);

            float w_value=std::exp(-u);
            weight.push_back(w_value);
            total_w+=w_value;
        }
    }

    for(unsigned int k=0;k<weight.size();++k)
        weight[k]/=total_w;

}

float anti_aliasing_table::dL(unsigned int k) const
{
    assert(k<displacement.size());
    return displacement[k];
}

float anti_aliasing_table::w(unsigned int kx,unsigned int ky) const
{
    int N=displacement.size();
    assert(ky+N*kx<weight.size());

    return weight[ky+N*kx];
}



}
