/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/



#ifndef ANTI_ALIASING_TABLE_HPP
#define ANTI_ALIASING_TABLE_HPP


#include <vector>

namespace cpe
{

/** \brief Compute weights for anti-aliasing rays


Example of use:

int N_aa=3;
float dL=0.25;
anti_aliasing_table aa(N_aa,dL);
for(int k_aa_x=0;k_aa_x<N_aa;++k_aa_x)
{
    for(int k_aa_y=0;k_aa_y<N_aa;++k_aa_y)
    {
       float dx=aa.dL(k_aa_x);
       float dy=aa.dL(k_aa_y);
       float w=aa.w(k_aa_x,k_aa_y);
    }
}


*/
class anti_aliasing_table
{
public:

    /** \brief Setup antialiasing for (NxN) rays with spacing=L*/
    anti_aliasing_table(int N,float L);

    /** \brief get spacing for ray k */
    float dL(unsigned int k) const;
    /** \brief get weight for ray k */
    float w(unsigned int kx,unsigned int ky) const;

private:

    /** \brief internal displacement */
    std::vector<float> displacement;
    /** \brief internal weight */
    std::vector<float> weight;
};

}


#endif
