/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef INTERSECTION_HPP
#define INTERSECTION_HPP

#include <vec3.hpp>
#include <iostream>

namespace cpe
{

/** \brief container for intersection data (position, normal, and relative position with respect to the ray) */
struct intersection
{
        /** \brief direct constructor */
        intersection(const vec3& x=vec3(),const vec3& n=vec3(0,0,1),double t=-1.0);

        /** \brief set intersection with data */
        void set(const vec3& x,const vec3& n,double t);
        /** \brief set intersection from an other one */
        void set(const intersection& inter);

        /** \brief position at the intersection */
        vec3 x;
        /** \brief normal at the intersection */
        vec3 n;
        /** \brief relative position of the intersection along the ray */
        double t;

};

std::ostream& operator<<(std::ostream& stream,const intersection& inter);

}

#endif
