/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <ray.hpp>

#include <cmath>
#include <assert.h>

namespace cpe
{
    ray::ray(const vec3& x0_data,const vec3& u_data)
        :x0(x0_data),u(u_data)
    {
        //check if direction is a unit vector
        float epsilon=1e-6;
        if(std::abs(u.norm2()-1.0)>epsilon)
            u=u.normalized();
    }


    vec3 ray::operator()(float t) const
    {
        return x0+t*u;
    }


    void ray::offset(const double& epsilon)
    {
        x0 += epsilon*u;
    }

    std::ostream& operator<<(std::ostream& stream,const ray& r)
    {
        stream<<r.x0<<" -> "<<r.u<<std::endl;
        return stream;
    }

    vec3 reflected_direction(const vec3& u,const vec3& n)
    {
        assert(std::abs(n.norm2()-1.0)<1e-6);
        assert(std::abs(u.norm2()-1.0)<1e-6);

        return u-2*u.dot(n)*n;
    }

}
