/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/





#ifndef RAY_HPP
#define RAY_HPP

#include <vec3.hpp>

namespace cpe
{

class camera;

/** \brief Straight ray (initial position + direction) used for ray-tracing algorithm */
class ray
{
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTOR
        // ********************************************* //
        // ********************************************* //

        /** \brief direct constructor (position+direction) */
        ray(const vec3& x0=vec3(),const vec3& u=vec3());


        // ********************************************* //
        // ********************************************* //
        //  RAY EVALUATION
        // ********************************************* //
        // ********************************************* //

        /** \brief evaluate 3D position at x0+t*u */
        vec3 operator()(float t) const;

        // ********************************************* //
        // ********************************************* //
        //  HELPER RAY CONSTRUCTION
        // ********************************************* //
        // ********************************************* //


        /** \brief offset the ray by epsilon in the direction of the normal */
        void offset(const double& epsilon=1e-3);



        /** \brief internal initial position of the ray */
        vec3 x0;
        /** \brief internal direction of the ray */
        vec3 u;
};

/** \brief exporting ray data */
std::ostream& operator<<(std::ostream& stream,const ray& r);

/** \brief return the reflected direction from a given ray with respect to the normal n
    \return u- 2<u,n> n
    \note u and n must have norm=1
*/
vec3 reflected_direction(const vec3& u,const vec3& n);

}



#endif
