/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <ray_tracer.hpp>


#include <ray_generator.hpp>
#include <pos2.hpp>
#include <intersection.hpp>
#include <plane.hpp>
#include <sphere.hpp>
#include <anti_aliasing_table.hpp>

namespace cpe
{

void render(image& image_parameter,const scene& scene_parameter)
{

    // ********************************************* //
    // ********************************************* //
    //
    // ALGORITHME:
    //
    // Pour tout pixel de l'image
    //    Generer rayon partant du centre de la camera vers pixel courant
    //    Calculer couleur associee a ce rayon
    //    Affecter couleur au pixel courant de l'image
    //
    // ********************************************* //
    // ********************************************* //

    const camera& cam=scene_parameter.get_camera();

    int Nx=image_parameter.Nx();
    int Ny=image_parameter.Ny();

    for(int kx=0;kx<Nx;++kx)
    {
        for(int ky=0;ky<Ny;++ky)
        {

            ray r=ray_generate(kx,ky,Nx,Ny,cam);

            color c=ray_trace(r,scene_parameter);

            image_parameter(pos2(kx,ky)) = c;
        }
    }
}



color ray_trace(const ray& r,const scene& scene_parameter)
{

    // ********************************************* //
    // ********************************************* //
    //
    // TO DO: Calculer la couleur associee a un rayon dans la scene 3D
    //
    // ********************************************* //
    // ********************************************* //


    //Le code suivant affecte la couleur de base de la premiere intersection trouvee
    //Ce code ne gère pas l'ombrage, l'illumination, ni les reflections.
    color color_final(0,0,0);

    intersection inter;
    int index=-1;
    bool is_intersection=compute_first_intersection(r,scene_parameter,inter,index);

    if(is_intersection==true && index>=0)
        color_final=scene_parameter.get_material(index).c;

    return color_final;
}


bool compute_first_intersection(const ray& r,const scene& scene_parameter,intersection& closest_inter,int& index)
{
    // ********************************************* //
    // ********************************************* //
    //
    // TO DO: Calculer l'intersection valide la plus proche
    //        Doit retourner vrai/faux si une intersection est trouvee ou non
    //        Doit mettre a jour closest_inter et index avec la premiere intersection
    //
    // ********************************************* //
    // ********************************************* //


    //Le code suivant renvoie la premiere intersection valide que l'on trouve dans l'ordre de stockage du vecteur
    //Ce code est a modifier afin de trouver la 1ere intersection dans l'espace 3D le long du rayon.
    intersection current_inter;

    int N_object=scene_parameter.N_object();
    for(int k=0;k<N_object;++k)
    {
        if(scene_parameter.get_object(k)->intersect(r,current_inter)==true)
        {
            index=k;
            closest_inter=current_inter;
            return true;
        }
    }

    return false;
}

bool is_in_shadow(const vec3& position,const vec3& light_position,const scene& scene_parameter)
{
    // ********************************************* //
    // ********************************************* //
    //
    // TO DO: Renvoie vrai/faux si la position
    //      courante est dans l'ombre d'autres objets
    //
    // Algorithme:
    //
    //    Generer rayon partant de position vers lumiere
    //    Si rayon intersecte un objet
    //       Si intersection + proche que la lumiere
    //          Renvoie VRAI
    //    Sinon
    //       Renvoie FAUX
    //
    // ********************************************* //
    // ********************************************* //

    return false;
}




}
