/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef RAY_TRACER_HPP
#define RAY_TRACER_HPP

#include <vector>

#include <image.hpp>
#include <scene.hpp>

namespace cpe
{

/** run rendering using ray-tracing algorithm given an image to write and a 3D scene */
void render(image& image_parameter,const scene& scene_parameter);

/** \brief Compute color associated to a single ray */
color ray_trace(const ray& r,const scene& scene_parameter);

/** \brief Compute the first intersection of a ray in the 3D scene.
    \return true if there is an intersection, false otherwise
    \param ray: the ray in the scene.
    \param scene_parameter: the 3D scene
    \param inter: the intersection parameter that is filled if there is an intersection.
    \param index: the index of the intersected object that is filled if there is an intersection.
*/
bool compute_first_intersection(const ray& r,const scene& scene_parameter,intersection& inter,int& index);

/** \brief Given a position in the 3D scene, and a position of a single light, the function
 * check if the position is in the shadow of other objects or not
    \return true if the object is in the shadow, return false otherwise
*/
bool is_in_shadow(const vec3& position,const vec3& light_position,const scene& scene_parameter);




}

#endif
