/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <camera.hpp>


namespace cpe
{
camera::camera(const vec3& center,const vec3& direction,const vec3& up_,float dist_screen_,float length_screen_)
    :p(center),dir(direction),up(up_),dist_screen(dist_screen_),length_screen(length_screen_)
{}

const vec3& camera::get_center() const
{
    return p;
}
const vec3& camera::get_direction() const
{
    return dir;
}
const vec3& camera::get_up() const
{
    return up;
}
float camera::get_dist_screen() const
{
    return dist_screen;
}
float camera::get_length_screen() const
{
    return length_screen;
}

vec3 screen_position(const camera& cam,float u,float v)
{

    const vec3& dir=cam.get_direction();
    const vec3& up=cam.get_up();
    const vec3& p=cam.get_center();
    float dist_screen=cam.get_dist_screen();
    float length_screen=cam.get_length_screen();

    vec3 right=dir.cross(up).normalized();
    vec3 x0=p+dir*dist_screen;

    vec3 tr_right=right*(u-0.5)*2.0*length_screen;
    vec3 tr_up=up*(v-0.5)*2.0*length_screen;

    vec3 current=x0+tr_right+tr_up;

    return current;
}

}
