/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef CAMERA_HPP
#define CAMERA_HPP

#include <vec3.hpp>

namespace cpe
{
class p2d;

/** \brief Storage of a camera and screen parameters */
class camera
{
    public:

        // ********************************************* //
        // ********************************************* //
        //  Constructor
        // ********************************************* //
        // ********************************************* //

        /** \brief direct constructor */
        camera(const vec3& center=vec3(0,0,-2),
               const vec3& direction=vec3(0,0,1),
               const vec3& up=vec3(0,1,0),
               float dist_screen=2.0,
               float length_screen=4.0);

        // ********************************************* //
        // ********************************************* //
        //  Access parameters
        // ********************************************* //
        // ********************************************* //

        /** \brief get the center of the camera */
        const vec3& get_center() const;
        /** \brief get the direction value */
        const vec3& get_direction() const;
        /** \brief get the up direction value */
        const vec3& get_up() const;
        /** \brief get the dist_screen value */
        float get_dist_screen() const;
        /** \brief get the length_of the screen value */
        float get_length_screen() const;

        // ********************************************* //
        // ********************************************* //
        //  Compute position on the screen
        // ********************************************* //
        // ********************************************* //

    private:

        /** \brief internal camera center */
        vec3 p;
        /** \brief direction of pointing camera */
        vec3 dir;
        /** \brief up direction of the camera */
        vec3 up;
        /** \brief internal screen distance to the center */
        float dist_screen;
        /** \brief length of the screen */
        float length_screen;
};

/** \brief return the 3D position of the pixel (u,v)
    \note (u,v) can be floating point, not only integers
*/
vec3 screen_position(const camera& cam,float u,float v);

}

#endif
