/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef MATERIAL_HPP
#define MATERIAL_HPP

#include <color.hpp>
#include <shading.hpp>

namespace cpe
{
/** \brief material associated to a given object (color+shading data) */
struct material
{
    /** \brief direct constructor (color+shading) */
    material(const color& c,const shading& shading_parameters,float reflection_power=0.5,double ior=1.0,double refraction_intensity=0.0);

    /** \brief original color of the object */
    color c;
    /** \brief shading parameter to compute illumination */
    shading shading_parameters;

    /** \brief the reflective power of an object */
    float reflection_power;
    /** \brief the refraction index of an object */
    double ior;
    /** \brief the refraction power of an object */
    double refraction_intensity;

};
}

#endif
