/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include <scene.hpp>

#include <material.hpp>
#include <camera.hpp>

#include <cstdlib>
#include <sstream>



namespace cpe
{
    scene::scene()
    {
        if(v_object.size()!=0)
        {
            clean_memory();
        }
        v_object.clear();
        v_light.clear();
    }

    void scene::clean_memory()
    {
        for(unsigned int k=0,N=v_object.size();k<N;++k)
        {
            delete v_object[k];
            v_object[k]=NULL;
        }
        v_object.clear();
    }

    void scene::add(const object3d* object,const material& material_data)
    {
        v_object.push_back(object);
        v_material.push_back(material_data);
    }
    void scene::add(const light& light_parameter)
    {
        v_light.push_back(light_parameter);
    }
    int scene::N_object() const
    {
        return v_object.size();
    }
    int scene::N_light() const
    {
        return v_light.size();
    }
    const object3d* scene::get_object(unsigned int k_object) const
    {

        if(k_object>=v_object.size())
        {
            //throw error
            std::ostringstream msg;msg<<"Object vector has size "<<v_object.size()<<" and you are trying to access element: "<<k_object;
            throw exception_scene3d(msg.str(),MACRO_EXCEPTION_PARAMETER);
        }

        return v_object[k_object];
    }
    const light& scene::get_light(unsigned int k_light) const
    {

        if(k_light>=v_light.size())
        {
            //throw error
            std::ostringstream msg;msg<<"Light vector has size "<<v_light.size()<<" and you are trying to access element: "<<k_light;
            throw exception_scene3d(msg.str(),MACRO_EXCEPTION_PARAMETER);
        }

        return v_light[k_light];
    }
    const material& scene::get_material(unsigned int k_material) const
    {

        if(k_material>=v_material.size())
        {
            //throw error
            std::ostringstream msg;msg<<"Object(/material) vector has size "<<v_object.size()<<" and you are trying to access element: "<<k_material;
            throw exception_scene3d(msg.str(),MACRO_EXCEPTION_PARAMETER);
        }

        return v_material[k_material];
    }

    void scene::set_camera(const camera& cam_parameter)
    {
        cam=cam_parameter;
    }
    const camera& scene::get_camera() const
    {return cam;}



}
