/*
**    TP CPE Lyon
**    Copyright (C) 2013 Damien Rohmer
**
**    This program is free software: you can redistribute it and/or modify
**    it under the terms of the GNU General Public License as published by
**    the Free Software Foundation, either version 3 of the License, or
**    (at your option) any later version.
**
**   This program is distributed in the hope that it will be useful,
**    but WITHOUT ANY WARRANTY; without even the implied warranty of
**    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**    GNU General Public License for more details.
**
**    You should have received a copy of the GNU General Public License
**    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#ifndef SCENE_HPP
#define SCENE_HPP

#include <vector>

#include <exception_cpe.hpp>
#include <light.hpp>
#include <object3d.hpp>
#include <color.hpp>
#include <light.hpp>
#include <camera.hpp>
#include <material.hpp>


namespace cpe
{





    /** \brief Container of object3d+material and light to define a 3D scene

         The class contains pointers. The free process is performed by the explicit call of clean_memory.
    */
    class scene
    {
    public:

        // ********************************************* //
        // ********************************************* //
        //  CONSTRUCTOR
        // ********************************************* //
        // ********************************************* //

        /** \brief create an empty scene */
        scene();

        // ********************************************* //
        // ********************************************* //
        //  ADD/DELETE OBJECTS
        // ********************************************* //
        // ********************************************* //

        /** \brief free pointers of objects 3D */
        void clean_memory();



        /** \brief add a generic object3d in the container and its material */
        void add(const object3d* obj,const material& material);
        /** \brief add a light in the scene */
        void add(const light& light_parameter);

        /** \brief get the number of object3d in the scene */
        int N_object() const;
        /** \brief get the number of light in the scene */
        int N_light() const;


        // ********************************************* //
        // ********************************************* //
        //  ADD/DELETE OBJECTS
        // ********************************************* //
        // ********************************************* //

        /** \brief get a given object */
        const object3d* get_object(unsigned int k_object) const;
        /** \brief get a given light */
        const light& get_light(unsigned int k_light) const;
        /** \brief get a given material */
        const material& get_material(unsigned int k_material) const;

        /** \brief set the camera */
        void set_camera(const camera& cam);
        /** \brief get the camera */
        const camera& get_camera() const;

    private:

        // ********************************************* //
        // ********************************************* //
        //  INTERNAL STORAGE
        // ********************************************* //
        // ********************************************* //

        /** \brief internal container for object3d */
        std::vector<const object3d*> v_object;
        /** \brief internal container for material */
        std::vector<material> v_material;
        /** \brief internal container for the lights */
        std::vector<light> v_light;
        /** \brief internal camera stored */
        camera cam;

    };





    /** \brief Special exception class for v3 */
    class exception_scene3d : public exception_cpe
    {
    public:

        /** \brief empty constructor */
        exception_scene3d():exception_cpe(){}
        /** \brief constructor with error information */
        exception_scene3d(const std::string& msg_data,const std::string& file_data,const std::string& caller_data,const int& line_data):exception_cpe(msg_data,file_data,caller_data,line_data){}
    };

}
#endif
