
/** TP 4ETI - CPE Lyon - 2013 */

#include <iostream>
#include <cstdlib>

#include <image.hpp>
#include <color.hpp>
#include <pos2.hpp>
#include <shading.hpp>

#include <ray.hpp>
#include <sphere.hpp>
#include <plane.hpp>
#include <intersection.hpp>
#include <ray_generator.hpp>
#include <scene.hpp>
#include <ray_tracer.hpp>

using namespace cpe;



int main(int argc,char *argv[])
{

    std::cout<<"****"<<std::endl;
    std::cout<<"run "<<argv[0]<<" with "<<argc-1<<" parameters ... \n"<<std::endl;

    try
    {
        //creation d'une image vide
        image im(250);

        //parametres de la scene 3D
        scene scene_parameter;
        //placement de la camera dans la scene 3D
        scene_parameter.set_camera(camera(vec3(0,0,-2),vec3(0,0,1),vec3(0,1,0),2.0,1.2));

        //parametres d'illumination par defaut
        shading shading_parameter;

        //ajout d'objets et de leur couleurs dans la scene 3D
        scene_parameter.add(new sphere(vec3(-0.5,1,2),0.9),
                            material(color(0.5,0.5,1.0),shading_parameter,0.5));

        scene_parameter.add(new sphere(vec3(1,1-0.2,1),0.2),
                            material(color(1,0,0),shading_parameter,0.9));

        scene_parameter.add(new sphere(vec3(+1.7,-0.2,5),0.3),
                            material(color(0.3,1,0),shading_parameter,0.9));

        scene_parameter.add(new plane(vec3(0,+1,0),vec3(0,-1,0)),
                            material(color(0.8,0.8,0.8),shading_parameter,0.1));


        //ajout d'une lumiere dans la scene 3D
        scene_parameter.add(light(vec3(15,-10,-10)));

        //Rendu de la scene par ray-tracing dans l'image
        render(im,scene_parameter);

        //liberation memoire des objets de la scene alloues dynamiquement
        scene_parameter.clean_memory();

        //ecriture de l'image au format ppm
        im.export_file("my_pic.ppm");

    }
    catch(exception_cpe e)
    {
        std::cout<<"**********************************"<<std::endl;
        std::cout<<"**********************************"<<std::endl;
        std::cout<<"Exception found"<<std::endl;
        std::cout<<e.info()<<std::endl;
        std::cout<<"**********************************"<<std::endl;
        std::cout<<"**********************************"<<std::endl;
        exit(1);
    }


    std::cout<<"Exit Main"<<std::endl;

    return 0;
}
